/**
 * 设备基础信息管理
 * @param Page
 * @returns
 */
import type { ActionType, ProColumns } from '@ant-design/pro-components';
import { ProTable } from '@ant-design/pro-components';
import { Button, Modal, Form, Input, message, Popconfirm, Tag, DatePicker as TDatePicker } from 'antd';
import { useRef, useState} from 'react';
import DeviceInfoAPI from '@/api/DeviceInfo';
import deviceProductAPI from '@/api/deviceInfo/services/deviceProduct';
import dayjs from 'dayjs'



const DatePicker: any = TDatePicker;

export default () => {
  //表单
  const [typeForm] = Form.useForm();
  const [modelTitle, setModelTitle] = useState<string>();
  //编辑/新增弹窗
  const [isModalOpen, setIsModalOpen] = useState(false);
  const actionRef = useRef<ActionType>();
  const columns: ProColumns<DeviceTypes.DeviceInfoOutput>[] = [
    {
      title: 'id',
      dataIndex: 'id',
      tip: '规则名称是唯一的 key',
      hideInSearch: true,
      hideInTable: true,
    },
    {
      title: '设备名称',
      dataIndex: 'name',
      ellipsis: true,
    },
    {
      title: '设备编号',
      dataIndex: 'code',
     
      ellipsis: true,
    },
    {
      title: '设备描述',
      dataIndex: 'describe',
      ellipsis: true,
      hideInSearch: true,
    },
    {
      title: '设备类型',
      dataIndex: 'deviceType',
      ellipsis: true,
      hideInSearch: true,
      valueType:'select',
      request:async ()=>{
        const b:MyResponse.Content =await deviceProductAPI.SelectList();
        console.log(b);
        
        return b.data;
      }
    },
    {
      title: '设备维保日期',
      dataIndex: 'repairDate',
      ellipsis: true,
      hideInSearch: true,
      render:(_,record)=>{
        return <>{dayjs(record.repairDate).format("YYYY-MM-DD")}</>
      }
    },
    {
      title: '操作',
      valueType: 'option',
      key: 'option',
      render: (_, record) => [
        <Button
          key="edit"
          type="primary" ghost
          onClick={async () => {
            setIsModalOpen(true);
            setModelTitle('编辑信息');
            const copyFormData = JSON.parse(JSON.stringify(record));
            copyFormData.repairDate =dayjs(record.repairDate)
            console.log('record',record);
            
            typeForm.setFieldsValue(copyFormData);
          }}
        >
          编辑
        </Button>,
        <Popconfirm
          key="del"
          title="确定要删除此条数据吗？"
          onConfirm={() => {
            const EnablejsonData: DeviceTypes.DeviceInfoDelInput = {
              id: record.id,
            };
            DeviceInfoAPI.Del(EnablejsonData).then((r) => {
              if (r.statusCode === 200) {
                message.success(r.statusCode === 200 ? '删除成功' : r.message);
                actionRef.current?.reload();
              }
            });
          }}
          onCancel={() => {
            message.info('已取消删除');
          }}
          okText="确认"
          cancelText="关闭"
        >
          <Button  type="primary" danger ghost>删除</Button>
        </Popconfirm>,
      ],
    },
  ];
  /*
   *表单提交
   */
  const OnSubmit = async (values: DeviceTypes.DeviceInfoOutput) => {

    if (values.id) {
      const response = await DeviceInfoAPI.Update(values);
      if (response.statusCode === 200) {
        message.success('修改成功');
        actionRef.current?.reload();
        setIsModalOpen(false);
      } else {
        message.error(JSON.stringify(response.errors)  || '修改失败');
      }
    } else {
      const response = await DeviceInfoAPI.Add(values);
      console.log('response', response);
      if (response.statusCode === 200) {
        message.success('添加成功');
        actionRef.current?.reload();
        setIsModalOpen(false);
      } else {
        message.error(JSON.stringify(response.errors)  || '添加失败');
      }
    }
  };
  return (
    <>
      <ProTable<DeviceTypes.DeviceInfoOutput>
        key="myTable"
        columns={columns}
        actionRef={actionRef}
        cardBordered
        request={async (params = {}) => {
          const jsonData: DeviceTypes.Page = {
            pageIndex: params.current || 1,
            pageSize: params.pageSize || 10,
            name: params.name || '',
            code: params.code
          };
          const response = await DeviceInfoAPI.PagedList(jsonData);
          if (response.statusCode === 200) {
            return {
              data: response.data.items,
              success: true,
              total: response.data.total,
            };
          } else {
            return {
              data: [],
              success: false,
              total: 0,
            };
          }
        }}
        rowKey="id"
        pagination={{
          pageSize: 10,
        }}
        dateFormatter="string"
        headerTitle="设备列表"
        toolBarRender={() => [
          <Button
            key="button"
            type="primary"
            onClick={() => {
              typeForm.resetFields();
              setIsModalOpen(true);
              setModelTitle('新增信息');
            }}
          >
            新建设备
          </Button>,
        ]}
      />
      <Modal
        key="01"
        width={800}
        title={<Tag color="#cd201f">{modelTitle}</Tag>}
        open={isModalOpen}
        onCancel={() => {
          setIsModalOpen(false);
        }}
        footer={[
          <Button
            key="back"
            onClick={() => {
              setIsModalOpen(false);
            }}
          >
            返回
          </Button>,
          <Button key="submit" form="deviceInfoForm" type="primary" htmlType="submit">
            提交
          </Button>,
        ]}
      >
        <Form
          form={typeForm}
          name="basic"
          id="deviceInfoForm"
      
          onFinish={OnSubmit}
          autoComplete="off"
        >
          <Form.Item label="id" name="id" hidden>
            <Input />
          </Form.Item>

          <Form.Item
            label="设备名称"
            name="name"
            rules={[{ required: true, message: '此项为必填项!' }]}
          >
            <Input />
          </Form.Item>
          <Form.Item label="设备编号" name="code" rules={[{ required: true, message: '此项为必填项!' }]} >
            <Input />
          </Form.Item>
          <Form.Item label="设备描述" name="describe"  >
            <Input />
          </Form.Item>
          <Form.Item label="设备类型" name="deviceType" rules={[{ required: true, message: '此项为必填项!' }]} >
            <Input />
          </Form.Item>
          {/* name="repairDate" */}
          <Form.Item label="设备维保日期" name="repairDate"  rules={[{ required: true, message: '此项为必填项!' }]} >
          <DatePicker ></DatePicker>
          </Form.Item>
        </Form>
      </Modal>
    </>
  );
};
